@@ -22,7 +22,9 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+
 import mock
+import yaml
 
 from flameclient import client as flame_client
 from flameclient import flame
@@ -92,6 +94,13 @@ class FakeSecurityGroup(FakeBase):
     name = 'name'
 
 
+class FakeServerGroup(FakeBase):
+    name = 'policy_group'
+    id = '1234'
+    policies = 'affinity'
+    members = ['12345']
+
+
 class FakeNeutronManager(object):
 
     def __init__(self):
@@ -141,6 +150,7 @@ class FakeNovaManager(object):
         self.groups = {}
         self.keypairs = [FakeKeypair(name='testkey',
                                      public_key='ssh-rsa XXXX')]
+        self.servergroups = [FakeServerGroup()]
 
     def keypair_list(self):
         return self.keypairs
@@ -154,6 +164,9 @@ class FakeNovaManager(object):
     def server_security_group_list(self, server):
         return self.groups.get(server.name, [])
 
+    def servergroup_list(self):
+        return self.servergroups
+
 
 class FakeCinderManager(object):
 
@@ -262,11 +275,12 @@ class BaseTestCase(base.TestCase):
         super(BaseTestCase, self).tearDown()
 
     def get_generator(self, exclude_servers, exclude_volumes,
-                      exclude_keypairs, generate_data):
+                      exclude_keypairs, generate_data, extract_ports):
         generator = flame.TemplateGenerator('x', 'x', 'x', 'x', True,
                                             'publicURL')
         generator.extract_vm_details(exclude_servers, exclude_volumes,
-                                     exclude_keypairs, generate_data)
+                                     exclude_keypairs, generate_data,
+                                     extract_ports)
         return generator
 
     def check_stackdata(self, resources, expected_resources):
@@ -290,6 +304,29 @@ class BaseTestCase(base.TestCase):
         self.assertEqual(expected_parameters, merged_parameters)
 
 
+class TemplateGenerationTest(BaseTestCase):
+    def test_heat_template_and_data_with_data(self):
+        generator = self.get_generator(False, False, False, True, False)
+        generator.extract_data()
+        out = yaml.load(generator.heat_template_and_data())
+        mandatory_keys = set(('environment', 'template', 'resources',
+                              'action', 'status'))
+        self.assertEqual(mandatory_keys, set(out.keys()))
+        self.assertEqual(generator.template, out['template'])
+        del out['environment']
+        del out['template']
+        self.assertEqual(generator.stack_data, out)
+
+    def test_heat_template_and_data_without_data(self):
+        generator = self.get_generator(False, False, False, False, False)
+        generator.extract_data()
+        out = yaml.load(generator.heat_template_and_data())
+        mandatory_keys = {'heat_template_version', 'resources', 'description',
+                          'parameters'}
+        self.assertEqual(mandatory_keys, set(out.keys()))
+        self.assertEqual(generator.template, out)
+
+
 class ClientTest(BaseTestCase):
     def setUp(self):
         super(ClientTest, self).setUp()
@@ -298,29 +335,27 @@ class ClientTest(BaseTestCase):
 
     def test_generate(self):
         out = self.c.generate(False, False, False, True)
-        self.assertIsInstance(out, tuple)
-        self.assertIsNotNone(out[1])
+        parsed_out = yaml.load(out)
+        self.assertIsInstance(parsed_out, dict)
 
     def test_generate_no_stack_data(self):
         out = self.c.generate(False, False, False, False)
-        self.assertIsInstance(out, tuple)
-        self.assertIsNotNone(out[0])
-        self.assertIsNone(out[1])
+        parsed_out = yaml.load(out)
+        self.assertIsInstance(parsed_out, dict)
+        self.assertNotIn('template', parsed_out.keys())
 
     def test_generate_contains_extract(self):
-        generator = self.get_generator(False, False, False, True)
-        out = self.c.generate(False, False, False, True)
-        generator.extract_data()
-        stack_data = generator.stack_data_template()
-        heat_template = generator.heat_template()
-        self.assertEqual(out, (heat_template, stack_data))
+        out = self.c.generate(False, False, False, True, False)
+        parsed_out = yaml.load(out)
+        self.assertIsInstance(parsed_out, dict)
+        self.assertIn('template', parsed_out.keys())
 
 
 class StackDataTests(BaseTestCase):
 
     def test_keypair(self):
         self.mock_nova.return_value = FakeNovaManager()
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected = {
             'key_0': {
@@ -337,7 +372,7 @@ class StackDataTests(BaseTestCase):
 
     def test_router(self):
         self.mock_neutron.return_value = FakeNeutronManager()
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected = {
             'router_0': {
@@ -360,8 +395,10 @@ class StackDataTests(BaseTestCase):
                          'external_gateway_info': {
                              'network_id': '8765',
                              'enable_snat': 'true'}}, ]
+        # This router has no port
+        fake.ports = []
         self.mock_neutron.return_value = fake
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected = {
             'router_0': {
@@ -414,7 +451,7 @@ class StackDataTests(BaseTestCase):
                          'id': '1111'}, ]
 
         self.mock_neutron.return_value = fake
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected = {
             'router_0': {
@@ -440,7 +477,7 @@ class StackDataTests(BaseTestCase):
 
     def test_network(self):
         self.mock_neutron.return_value = FakeNeutronManager()
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected = {
             'network_0': {
@@ -460,7 +497,7 @@ class StackDataTests(BaseTestCase):
         fake.networks[0]['router:external'] = True
         self.mock_neutron.return_value = fake
 
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         self.check_stackdata(generator._extract_networks(), {})
 
@@ -479,7 +516,7 @@ class StackDataTests(BaseTestCase):
                          'id': '1111'}, ]
         self.mock_neutron.return_value = fake
 
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected = {
             'subnet_0': {
@@ -505,7 +542,7 @@ class StackDataTests(BaseTestCase):
                              'id': '2222'}, ]
         self.mock_neutron.return_value = fake
 
-        generator = self.get_generator(True, False, False, True)
+        generator = self.get_generator(True, False, False, True, False)
 
         expected = {
             'floatingip_0': {
@@ -539,7 +576,7 @@ class StackDataTests(BaseTestCase):
                         'id': '1234'}, ]
         self.mock_neutron.return_value = fake
 
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected = {
             'security_group_0': {
@@ -573,14 +610,14 @@ class StackDataTests(BaseTestCase):
                         'id': '1234'}, ]
         self.mock_neutron.return_value = fake
 
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         self.check_stackdata(generator._extract_secgroups(), {})
 
     def test_volume(self):
         self.mock_cinder.return_value = FakeCinderManager()
 
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected = {
             'volume_0': {
@@ -598,7 +635,7 @@ class StackDataTests(BaseTestCase):
     def test_server(self):
         self.mock_nova.return_value = FakeNovaManager()
 
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected = {
             'server_0': {
@@ -625,7 +662,7 @@ class StackDataTests(BaseTestCase):
         self.mock_neutron.return_value = fake_neutron
         self.mock_nova.return_value = fake_nova
 
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected = {
             'server_0': {
@@ -640,12 +677,29 @@ class StackDataTests(BaseTestCase):
         }
         self.check_stackdata(generator._extract_servers(), expected)
 
+    def test_servergroup(self):
+        self.mock_nova.return_value = FakeNovaManager()
+        generator = self.get_generator(False, False, False, True, False)
+
+        expected = {
+            'servergroup_0': {
+                'action': 'CREATE',
+                'metadata': {},
+                'name': 'servergroup_0',
+                'resource_data': {},
+                'resource_id': '1234',
+                'status': 'COMPLETE',
+                'type': 'OS::Nova::ServerGroup'
+            }
+        }
+        self.check_stackdata(generator._extract_servergroups(), expected)
+
 
 class NetworkTests(BaseTestCase):
 
     def test_keypair(self):
         self.mock_nova.return_value = FakeNovaManager()
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected = {
             'key_0': {
@@ -660,7 +714,8 @@ class NetworkTests(BaseTestCase):
 
     def test_router(self):
         self.mock_neutron.return_value = FakeNeutronManager()
-        generator = self.get_generator(False, False, False, True)
+        self.mock_neutron.return_value.ports = []
+        generator = self.get_generator(False, False, False, True, False)
 
         expected = {
             'router_0': {
@@ -675,6 +730,7 @@ class NetworkTests(BaseTestCase):
 
     def test_router_with_external_gateway(self):
         fake = FakeNeutronManager()
+        fake.ports = []
         fake.routers = [{'name': 'myrouter',
                          'id': '1234',
                          'admin_state_up': 'true',
@@ -682,7 +738,7 @@ class NetworkTests(BaseTestCase):
                              'network_id': '8765',
                              'enable_snat': 'true'}}, ]
         self.mock_neutron.return_value = fake
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'router_0_external_network': {
@@ -741,7 +797,7 @@ class NetworkTests(BaseTestCase):
                          'id': '1111'}, ]
 
         self.mock_neutron.return_value = fake
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected = {
             'router_0': {
@@ -763,7 +819,7 @@ class NetworkTests(BaseTestCase):
 
     def test_network(self):
         self.mock_neutron.return_value = FakeNeutronManager()
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected = {
             'network_0': {
@@ -782,7 +838,7 @@ class NetworkTests(BaseTestCase):
         fake.networks[0]['router:external'] = True
         self.mock_neutron.return_value = fake
 
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         self.check_template(generator._extract_networks(), {})
 
@@ -801,7 +857,7 @@ class NetworkTests(BaseTestCase):
                          'id': '1111'}, ]
         self.mock_neutron.return_value = fake
 
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected = {
             'subnet_0': {
@@ -832,7 +888,7 @@ class NetworkTests(BaseTestCase):
                              'id': '2222'}, ]
         self.mock_neutron.return_value = fake
 
-        generator = self.get_generator(True, False, False, False)
+        generator = self.get_generator(True, False, False, False, False)
 
         expected_parameters = {
             'external_network_for_floating_ip_0': {
@@ -913,7 +969,7 @@ class NetworkTests(BaseTestCase):
                         'id': '1234'}, ]
         self.mock_neutron.return_value = fake
 
-        generator = self.get_generator(False, False, False, False)
+        generator = self.get_generator(False, False, False, False, False)
 
         expected = {
             'security_group_0': {
@@ -1023,7 +1079,7 @@ class NetworkTests(BaseTestCase):
                         'id': '1111'}, ]
         self.mock_neutron.return_value = fake
 
-        generator = self.get_generator(False, False, False, False)
+        generator = self.get_generator(False, False, False, False, False)
 
         expected = {
             'security_group_0': {
@@ -1179,7 +1235,7 @@ class NetworkTests(BaseTestCase):
                         'id': '2222'}, ]
         self.mock_neutron.return_value = fake
 
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected = {
             'security_group_0': {
@@ -1267,7 +1323,7 @@ class VolumeTests(BaseTestCase):
         self.mock_cinder.return_value = self.fake
 
     def test_basic(self):
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'volume_0_volume_type': {
@@ -1292,7 +1348,7 @@ class VolumeTests(BaseTestCase):
 
     def test_basic_unnamed(self):
         self.fake.volumes = [FakeUnnamedVolume(), ]
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'volume_0_volume_type': {
@@ -1315,7 +1371,7 @@ class VolumeTests(BaseTestCase):
 
     def test_source_volid_external(self):
         self.fake.volumes = [FakeVolume(source_volid=5678), ]
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'volume_0_source_volid': {
@@ -1345,7 +1401,7 @@ class VolumeTests(BaseTestCase):
     def test_source_volid_included(self):
         self.fake.volumes = [FakeVolume(source_volid=5678),
                              FakeVolume(id=5678)]
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'volume_0_volume_type': {
@@ -1398,7 +1454,7 @@ class VolumeTests(BaseTestCase):
             'size': '25'}
         self.fake.volumes = [FakeVolume(bootable='true',
                                         volume_image_metadata=metadata), ]
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'volume_0_volume_type': {
@@ -1429,7 +1485,7 @@ class VolumeTests(BaseTestCase):
 
     def test_snapshot_id(self):
         self.fake.volumes = [FakeVolume(snapshot_id=5678), ]
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'volume_0_snapshot_id': {
@@ -1460,7 +1516,7 @@ class VolumeTests(BaseTestCase):
 
     def test_volume_type(self):
         self.fake.volumes = [FakeVolume(volume_type='isci'), ]
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'volume_0_volume_type': {
@@ -1485,7 +1541,7 @@ class VolumeTests(BaseTestCase):
 
     def test_metadata(self):
         self.fake.volumes = [FakeVolume(metadata={'key': 'value'}), ]
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'volume_0_volume_type': {
@@ -1518,7 +1574,7 @@ class ServerTests(BaseTestCase):
         self.mock_nova.return_value = self.fake
 
     def test_basic(self):
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'server_0_flavor': {
@@ -1549,7 +1605,7 @@ class ServerTests(BaseTestCase):
 
     def test_keypair(self):
         self.fake.servers = [FakeServer(key_name='testkey')]
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'server_0_flavor': {
@@ -1595,7 +1651,7 @@ class ServerTests(BaseTestCase):
                                           bootable='true')]
         self.mock_cinder.return_value = fake_cinder
 
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'server_0_flavor': {
@@ -1633,7 +1689,7 @@ class ServerTests(BaseTestCase):
         server_args = {"id": 777,
                        "os-extended-volumes:volumes_attached": [{'id': 5678}]}
         self.fake.servers = [FakeServer(**server_args), ]
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'server_0_flavor': {
@@ -1673,7 +1729,7 @@ class ServerTests(BaseTestCase):
                                 "description": "Group"}, ]
         self.mock_neutron.return_value = fake_neutron
         self.fake.groups = {'server1': [FakeSecurityGroup(), ]}
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'server_0_flavor': {
@@ -1710,7 +1766,7 @@ class ServerTests(BaseTestCase):
 
     def test_config_drive(self):
         self.fake.servers = [FakeServer(config_drive="True"), ]
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'server_0_flavor': {
@@ -1742,7 +1798,7 @@ class ServerTests(BaseTestCase):
 
     def test_metadata(self):
         self.fake.servers = [FakeServer(metadata={"key": "value"}), ]
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'server_0_flavor': {
@@ -1789,7 +1845,7 @@ class ServerTests(BaseTestCase):
         self.mock_neutron.return_value = fake_neutron
         addresses = {"private": [{"addr": "10.0.0.2"}]}
         self.fake.servers = [FakeServer(addresses=addresses)]
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'server_0_flavor': {
@@ -1833,7 +1889,7 @@ class ServerTests(BaseTestCase):
         server_args = {"id": 777,
                        "os-extended-volumes:volumes_attached": [{'id': 5678}]}
         self.fake.servers = [FakeServer(**server_args), ]
-        generator = self.get_generator(False, True, False, False)
+        generator = self.get_generator(False, True, False, False, False)
 
         expected_parameters = {
             'server_0_flavor': {
@@ -1871,6 +1927,40 @@ class ServerTests(BaseTestCase):
         self.check_template(generator._extract_servers(), expected_resources,
                             expected_parameters)
 
+    def test_servergroup(self):
+        self.fake.servers = [FakeServer()]
+        self.fake.servers[0].id = '12345'
+        generator = self.get_generator(False, False, False, True, False)
+
+        expected_parameters = {
+            'server_0_flavor': {
+                'default': 'm1.small',
+                'description': 'Flavor to use for server server_0',
+                'type': 'string'
+            },
+            'server_0_image': {
+                'description': 'Image to use to boot server server_0',
+                'default': '3333',
+                'type': 'string'
+            }
+        }
+        expected_resources = {
+            'server_0': {
+                'type': 'OS::Nova::Server',
+                'properties': {
+                    'name': 'server1',
+                    'diskConfig': 'MANUAL',
+                    'flavor': {'get_param': 'server_0_flavor'},
+                    'image': {'get_param': 'server_0_image'},
+                    'key_name': {'get_resource': 'key_0'},
+                    'scheduler_hints': {'group':
+                                        {'get_resource': 'servergroup_0'}}
+                }
+            }
+        }
+        self.check_template(generator._extract_servers(), expected_resources,
+                            expected_parameters)
+
 
 class GenerationTests(BaseTestCase):
 
@@ -1882,7 +1972,7 @@ class GenerationTests(BaseTestCase):
 
     def test_generation(self):
 
-        generator = self.get_generator(False, False, False, True)
+        generator = self.get_generator(False, False, False, True, False)
 
         expected_parameters = {
             'server_0_flavor': {
@@ -1934,6 +2024,12 @@ class GenerationTests(BaseTestCase):
                 },
                 'type': 'OS::Nova::Server'
             },
+            'servergroup_0': {
+                'properties': {
+                    'name': 'policy_group',
+                    'policies': 'affinity'},
+                'type': 'OS::Nova::ServerGroup'
+            },
             'volume_0': {
                 'properties': {
                     'description': 'Description',
@@ -1982,6 +2078,15 @@ class GenerationTests(BaseTestCase):
                 'status': 'COMPLETE',
                 'type': 'OS::Nova::Server'
             },
+            'servergroup_0': {
+                'action': 'CREATE',
+                'metadata': {},
+                'name': 'servergroup_0',
+                'resource_data': {},
+                'resource_id': '1234',
+                'status': 'COMPLETE',
+                'type': 'OS::Nova::ServerGroup'
+            },
             'volume_0': {
                 'action': 'CREATE',
                 'metadata': {},
@@ -1999,7 +2104,7 @@ class GenerationTests(BaseTestCase):
 
     def test_generation_exclude_servers(self):
 
-        generator = self.get_generator(True, False, False, True)
+        generator = self.get_generator(True, False, False, True, False)
 
         expected_parameters = {
             'volume_0_volume_type': {
@@ -2032,6 +2137,13 @@ class GenerationTests(BaseTestCase):
                 },
                 'type': 'OS::Neutron::Router'
             },
+            'servergroup_0': {
+                'properties': {
+                    'name': 'policy_group',
+                    'policies': 'affinity'
+                },
+                'type': 'OS::Nova::ServerGroup'
+            },
             'volume_0': {
                 'properties': {
                     'description': 'Description',
@@ -2071,6 +2183,15 @@ class GenerationTests(BaseTestCase):
                 'status': 'COMPLETE',
                 'type': 'OS::Neutron::Router'
             },
+            'servergroup_0': {
+                'action': 'CREATE',
+                'metadata': {},
+                'name': 'servergroup_0',
+                'resource_data': {},
+                'resource_id': '1234',
+                'status': 'COMPLETE',
+                'type': 'OS::Nova::ServerGroup'
+            },
             'volume_0': {
                 'action': 'CREATE',
                 'metadata': {},
@@ -2089,7 +2210,7 @@ class GenerationTests(BaseTestCase):
 
     def test_generation_exclude_volumes(self):
 
-        generator = self.get_generator(False, True, False, True)
+        generator = self.get_generator(False, True, False, True, False)
 
         expected_parameters = {
             'server_0_flavor': {
@@ -2136,6 +2257,12 @@ class GenerationTests(BaseTestCase):
                 },
                 'type': 'OS::Nova::Server'
             },
+            'servergroup_0': {
+                'properties': {
+                    'name': 'policy_group',
+                    'policies': 'affinity'},
+                'type': 'OS::Nova::ServerGroup'
+            }
         }
 
         expected_data = {
@@ -2174,6 +2301,15 @@ class GenerationTests(BaseTestCase):
                 'resource_id': '1234',
                 'status': 'COMPLETE',
                 'type': 'OS::Nova::Server'
+            },
+            'servergroup_0': {
+                'action': 'CREATE',
+                'metadata': {},
+                'name': 'servergroup_0',
+                'resource_data': {},
+                'resource_id': '1234',
+                'status': 'COMPLETE',
+                'type': 'OS::Nova::ServerGroup'
             }
         }
 
@@ -2184,7 +2320,7 @@ class GenerationTests(BaseTestCase):
 
     def test_generation_exclude_keypairs(self):
 
-        generator = self.get_generator(False, False, True, True)
+        generator = self.get_generator(False, False, True, True, False)
 
         expected_parameters = {
             'server_0_flavor': {
@@ -2234,6 +2370,13 @@ class GenerationTests(BaseTestCase):
                 },
                 'type': 'OS::Nova::Server'
             },
+            'servergroup_0': {
+                'properties': {
+                    'name': 'policy_group',
+                    'policies': 'affinity'
+                },
+                'type': 'OS::Nova::ServerGroup'
+            },
             'volume_0': {
                 'properties': {
                     'description': 'Description',
@@ -2273,6 +2416,15 @@ class GenerationTests(BaseTestCase):
                 'status': 'COMPLETE',
                 'type': 'OS::Nova::Server'
             },
+            'servergroup_0': {
+                'action': 'CREATE',
+                'metadata': {},
+                'name': 'servergroup_0',
+                'resource_data': {},
+                'resource_id': '1234',
+                'status': 'COMPLETE',
+                'type': 'OS::Nova::ServerGroup'
+            },
             'volume_0': {
                 'action': 'CREATE',
                 'metadata': {},
@@ -2291,7 +2443,7 @@ class GenerationTests(BaseTestCase):
 
     def test_generation_exclude_servers_and_volumes(self):
 
-        generator = self.get_generator(True, True, False, True)
+        generator = self.get_generator(True, True, False, True, False)
 
         expected_parameters = {}
         expected_resources = {
@@ -2317,6 +2469,13 @@ class GenerationTests(BaseTestCase):
                 },
                 'type': 'OS::Neutron::Router'
             },
+            'servergroup_0': {
+                'properties': {
+                    'name': 'policy_group',
+                    'policies': 'affinity'
+                },
+                'type': 'OS::Nova::ServerGroup'
+            }
         }
 
         expected_data = {
@@ -2346,6 +2505,15 @@ class GenerationTests(BaseTestCase):
                 'resource_id': '1234',
                 'status': 'COMPLETE',
                 'type': 'OS::Neutron::Router'
+            },
+            'servergroup_0': {
+                'action': 'CREATE',
+                'metadata': {},
+                'name': 'servergroup_0',
+                'resource_data': {},
+                'resource_id': '1234',
+                'status': 'COMPLETE',
+                'type': 'OS::Nova::ServerGroup'
             }
         }
         generator.extract_data()
@@ -2355,7 +2523,7 @@ class GenerationTests(BaseTestCase):
 
     def test_generation_exclude_servers_volumes_keypairs(self):
 
-        generator = self.get_generator(True, True, True, True)
+        generator = self.get_generator(True, True, True, True, False)
 
         expected_parameters = {}
         expected_resources = {
@@ -2373,8 +2541,16 @@ class GenerationTests(BaseTestCase):
                     'name': 'myrouter'
                 },
                 'type': 'OS::Neutron::Router'
+            },
+            'servergroup_0': {
+                'properties': {
+                    'name': 'policy_group',
+                    'policies': 'affinity'
+                },
+                'type': 'OS::Nova::ServerGroup'
             }
         }
+
         expected_data = {
             'network_0': {
                 'action': 'CREATE',
@@ -2393,6 +2569,15 @@ class GenerationTests(BaseTestCase):
                 'resource_id': '1234',
                 'status': 'COMPLETE',
                 'type': 'OS::Neutron::Router'
+            },
+            'servergroup_0': {
+                'action': 'CREATE',
+                'metadata': {},
+                'name': 'servergroup_0',
+                'resource_data': {},
+                'resource_id': '1234',
+                'status': 'COMPLETE',
+                'type': 'OS::Nova::ServerGroup'
             }
         }
 
